Class: Optimist::Parser

Inherits:
Object
  • Object
show all
Defined in:
lib/optimist_with_insert_blanks.rb

Overview

The commandline parser. In typical usage, the methods in this class will be handled internally by Optimist::options. In this case, only the #opt, #banner and #version, #depends, and #conflicts methods will typically be called.

If you want to instantiate this class yourself (for more complicated argument-parsing logic), call #parse to actually produce the output hash, and consider calling it from within Optimist::with_standard_exception_handling.

Constant Summary collapse

INVALID_SHORT_ARG_REGEX =

:nodoc:

/[\d-]/

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(*a, &b) ⇒ Parser

Initializes the parser, and instance-evaluates any block given.



152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
# File 'lib/optimist_with_insert_blanks.rb', line 152

def initialize(*a, &b)
  @version = nil
  @leftovers = []
  @specs = {}
  @long = {}
  @short = {}
  @order = []
  @constraints = []
  @stop_words = []
  @stop_on_unknown = false
  @educate_on_error = false
  @synopsis = nil
  @usage = nil

  # instance_eval(&b) if b # can't take arguments
  cloaker(&b).bind(self).call(*a) if b
end

Instance Attribute Details

#ignore_invalid_optionsObject

A flag that determines whether or not to raise an error if the parser is passed one or more

options that were not registered ahead of time.  If 'true', then the parser will simply
ignore options that it does not recognize.


149
150
151
# File 'lib/optimist_with_insert_blanks.rb', line 149

def ignore_invalid_options
  @ignore_invalid_options
end

#leftoversObject (readonly)

The values from the commandline that were not interpreted by #parse.



140
141
142
# File 'lib/optimist_with_insert_blanks.rb', line 140

def leftovers
  @leftovers
end

#specsObject (readonly)

The complete configuration hashes for each option. (Mainly useful for testing.)



144
145
146
# File 'lib/optimist_with_insert_blanks.rb', line 144

def specs
  @specs
end

Class Method Details

.register(lookup, klass) ⇒ Object

The Option subclasses are responsible for registering themselves using this function.



114
115
116
# File 'lib/optimist_with_insert_blanks.rb', line 114

def self.register(lookup, klass)
  @registry[lookup.to_sym] = klass
end

.registry_getopttype(type) ⇒ Object

Gets the class from the registry. Can be given either a class-name, e.g. Integer, a string, e.g “integer”, or a symbol, e.g :integer



120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
# File 'lib/optimist_with_insert_blanks.rb', line 120

def self.registry_getopttype(type)
  return nil unless type

  if type.respond_to?(:name)
    type = type.name
    lookup = type.downcase.to_sym
  else
    lookup = type.to_sym
  end
  unless @registry.has_key?(lookup)
    raise ArgumentError,
          "Unsupported argument type '#{type}', registry lookup '#{lookup}'"
  end

  @registry[lookup].new
end

Instance Method Details

Adds text to the help display. Can be interspersed with calls to #opt to build a multi-section help page.



255
256
257
# File 'lib/optimist_with_insert_blanks.rb', line 255

def banner(s)
  @order << [:text, s]
end

#conflicts(*syms) ⇒ Object

Marks two (or more!) options as conflicting.



269
270
271
272
# File 'lib/optimist_with_insert_blanks.rb', line 269

def conflicts(*syms)
  syms.each { |sym| raise ArgumentError, "unknown option '#{sym}'" unless @specs[sym] }
  @constraints << [:conflicts, syms]
end

#depends(*syms) ⇒ Object

Marks two (or more!) options as requiring each other. Only handles undirected (i.e., mutual) dependencies. Directed dependencies are better modeled with Optimist::die.



263
264
265
266
# File 'lib/optimist_with_insert_blanks.rb', line 263

def depends(*syms)
  syms.each { |sym| raise ArgumentError, "unknown option '#{sym}'" unless @specs[sym] }
  @constraints << [:depends, syms]
end

#die(arg, msg = nil, error_code = nil) ⇒ Object

The per-parser version of Optimist::die (see that for documentation).



536
537
538
539
540
541
542
543
544
545
546
547
548
549
550
551
552
553
# File 'lib/optimist_with_insert_blanks.rb', line 536

def die(arg, msg = nil, error_code = nil)
  if msg.is_a?(Integer)
    error_code = msg
    msg = nil
  end
  if msg
    warn "Error: argument --#{@specs[arg].long} #{msg}."
  else
    warn "Error: #{arg}."
  end
  if @educate_on_error
    $stderr.puts
    educate $stderr
  else
    warn 'Try --help for help.'
  end
  exit(error_code || -1)
end

#educate(stream = $stdout) ⇒ Object

Print the help message to stream.



465
466
467
468
469
470
471
472
473
474
475
476
477
478
479
480
481
482
483
484
485
486
487
488
489
490
491
492
493
494
495
496
497
498
# File 'lib/optimist_with_insert_blanks.rb', line 465

def educate(stream = $stdout)
  width # HACK: calculate it now; otherwise we have to be careful not to
  # call this unless the cursor's at the beginning of a line.

  left = {}
  @specs.each { |name, spec| left[name] = spec.educate }

  leftcol_width = left.values.map(&:length).max || 0
  rightcol_start = leftcol_width + 6 # spaces

  unless @order.size > 0 && @order.first.first == :text
    command_name = File.basename($0).gsub(/\.[^.]+$/, '')
    stream.puts "Usage: #{command_name} #{@usage}\n" if @usage
    stream.puts "#{@synopsis}\n" if @synopsis
    stream.puts if @usage || @synopsis
    stream.puts "#{@version}\n" if @version
    stream.puts 'Options:'
  end

  @order.each do |what, opt|
    if what == :text
      stream.puts wrap(opt)
      next
    end

    spec = @specs[opt]
    stream.printf "  %-#{leftcol_width}s    ", left[opt]
    desc = spec.description_with_default

    stream.puts wrap(desc, width: width - rightcol_start - 1, prefix: rightcol_start)
    # HACK: by chrisl - put a  blank line between each option in the help, if desired
    stream.puts if @insert_blanks
  end
end

#educate_on_errorObject

Instead of displaying “Try –help for help.” on an error display the usage (via educate)



310
311
312
# File 'lib/optimist_with_insert_blanks.rb', line 310

def educate_on_error
  @educate_on_error = true
end

#either(*syms) ⇒ Object

Marks two (or more!) options as required but mutually exclusive.



275
276
277
278
279
# File 'lib/optimist_with_insert_blanks.rb', line 275

def either(*syms)
  syms.each { |sym| raise ArgumentError, "unknown option '#{sym}'" unless @specs[sym] }
  @constraints << [:conflicts, syms]
  @constraints << [:either, syms]
end

#insert_blanksObject

Put a blank line between the help output for each option HACK by chrisl - put a blank line between each option in the help, if desired



296
297
298
# File 'lib/optimist_with_insert_blanks.rb', line 296

def insert_blanks
  @insert_blanks = true
end

#opt(name, desc = '', opts = {}, &b) ⇒ Object

Define an option. name is the option name, a unique identifier for the option that you will use internally, which should be a symbol or a string. desc is a string description which will be displayed in help messages.

Takes the following optional arguments:

:long

Specify the long form of the argument, i.e. the form with two dashes. If unspecified, will be automatically derived based on the argument name by turning the name option into a string, and replacing any _’s by -‘s.

:short

Specify the short form of the argument, i.e. the form with one dash. If unspecified, will be automatically derived from name. Use :none: to not have a short value.

:type

Require that the argument take a parameter or parameters of type type. For a single parameter, the value can be a member of SINGLE_ARG_TYPES, or a corresponding Ruby class (e.g. Integer for :int). For multiple-argument parameters, the value can be any member of MULTI_ARG_TYPES constant. If unset, the default argument type is :flag, meaning that the argument does not take a parameter. The specification of :type is not necessary if a :default is given.

:default

Set the default value for an argument. Without a default value, the hash returned by #parse (and thus Optimist::options) will have a nil value for this key unless the argument is given on the commandline. The argument type is derived automatically from the class of the default value given, so specifying a :type is not necessary if a :default is given. (But see below for an important caveat when :multi: is specified too.) If the argument is a flag, and the default is set to true, then if it is specified on the the commandline the value will be false.

:required

If set to true, the argument must be provided on the commandline.

:multi

If set to true, allows multiple occurrences of the option on the commandline. Otherwise, only a single instance of the option is allowed. (Note that this is different from taking multiple parameters. See below.)

Note that there are two types of argument multiplicity: an argument can take multiple values, e.g. “–arg 1 2 3”. An argument can also be allowed to occur multiple times, e.g. “–arg 1 –arg 2”.

Arguments that take multiple values should have a :type parameter drawn from MULTI_ARG_TYPES (e.g. :strings), or a :default: value of an array of the correct type (e.g. [String]). The value of this argument will be an array of the parameters on the commandline.

Arguments that can occur multiple times should be marked with :multi => true. The value of this argument will also be an array. In contrast with regular non-multi options, if not specified on the commandline, the default value will be [], not nil.

These two attributes can be combined (e.g. :type => :strings, :multi => true), in which case the value of the argument will be an array of arrays.

There’s one ambiguous case to be aware of: when :multi: is true and a :default is set to an array (of something), it’s ambiguous whether this is a multi-value argument as well as a multi-occurrence argument. In thise case, Optimist assumes that it’s not a multi-value argument. If you want a multi-value, multi-occurrence argument with a default value, you must specify :type as well.

Raises:

  • (ArgumentError)


210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
# File 'lib/optimist_with_insert_blanks.rb', line 210

def opt(name, desc = '', opts = {}, &b)
  opts[:callback] ||= b if block_given?
  opts[:desc] ||= desc

  o = Option.create(name, desc, opts)

  raise ArgumentError, "you already have an argument named '#{name}'" if @specs.member? o.name

  if @long[o.long]
    raise ArgumentError,
          "long option name #{o.long.inspect} is already taken; please specify a (different) :long"
  end
  if @short[o.short]
    raise ArgumentError,
          "short option name #{o.short.inspect} is already taken; please specify a (different) :short"
  end

  @long[o.long] = o.name
  @short[o.short] = o.name if o.short?
  @specs[o.name] = o
  @order << [:opt, o.name]
end

#parse(cmdline = ARGV) ⇒ Object

Parses the commandline. Typically called by Optimist::options, but you can call it directly if you need more control.

throws CommandlineError, HelpNeeded, and VersionNeeded exceptions.

Raises:



318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
400
401
402
403
404
405
406
407
408
409
410
411
412
413
414
415
416
417
418
419
420
421
422
423
424
425
426
427
428
429
430
431
432
433
434
435
436
437
438
439
440
441
442
443
444
445
446
447
448
449
450
451
452
453
454
455
456
457
458
459
460
461
462
# File 'lib/optimist_with_insert_blanks.rb', line 318

def parse(cmdline = ARGV)
  vals = {}
  required = {}

  opt :version, 'Print version and exit' if @version && !(@specs[:version] || @long['version'])
  opt :help, 'Show this message' unless @specs[:help] || @long['help']

  @specs.each do |sym, opts|
    required[sym] = true if opts.required?
    vals[sym] = opts.default
    vals[sym] = [] if opts.multi && !opts.default # multi arguments default to [], not nil
  end

  resolve_default_short_options!

  ## resolve symbols
  given_args = {}
  @leftovers = each_arg cmdline do |arg, params|
    ## handle --no- forms
    arg, negative_given = if arg =~ /^--no-([^-]\S*)$/
                            ["--#{::Regexp.last_match(1)}", true]
                          else
                            [arg, false]
                          end

    sym = case arg
          when /^-([^-])$/      then @short[::Regexp.last_match(1)]
          when /^--([^-]\S*)$/  then @long[::Regexp.last_match(1)] || @long["no-#{::Regexp.last_match(1)}"]
          else raise CommandlineError, "invalid argument syntax: '#{arg}'"
          end

    sym = nil if arg =~ /--no-/ # explicitly invalidate --no-no- arguments

    next nil if ignore_invalid_options && !sym
    raise CommandlineError, "unknown argument '#{arg}'" unless sym

    if given_args.include?(sym) && !@specs[sym].multi?
      raise CommandlineError, "option '#{arg}' specified multiple times"
    end

    given_args[sym] ||= {}
    given_args[sym][:arg] = arg
    given_args[sym][:negative_given] = negative_given
    given_args[sym][:params] ||= []

    # The block returns the number of parameters taken.
    num_params_taken = 0

    unless params.empty?
      if @specs[sym].single_arg?
        given_args[sym][:params] << params[0, 1]  # take the first parameter
        num_params_taken = 1
      elsif @specs[sym].multi_arg?
        given_args[sym][:params] << params        # take all the parameters
        num_params_taken = params.size
      end
    end

    num_params_taken
  end

  ## check for version and help args
  raise VersionNeeded if given_args.include? :version
  raise HelpNeeded if given_args.include? :help

  ## check constraint satisfaction
  @constraints.each do |type, syms|
    constraint_sym = syms.find { |sym| given_args[sym] }

    case type
    when :depends
      next unless constraint_sym

      syms.each do |sym|
        unless given_args.include? sym
          raise CommandlineError,
                "--#{@specs[constraint_sym].long} requires --#{@specs[sym].long}"
        end
      end
    when :conflicts
      next unless constraint_sym

      syms.each do |sym|
        if given_args.include?(sym) && (sym != constraint_sym)
          raise CommandlineError,
                "--#{@specs[constraint_sym].long} conflicts with --#{@specs[sym].long}"
        end
      end
    when :either
      if (syms & given_args.keys).size != 1
        raise CommandlineError, "one of #{syms.map do |sym|
                                            "--#{@specs[sym].long}"
                                          end.join(', ')} is required"
      end
    end
  end

  required.each do |sym, _val|
    raise CommandlineError, "option --#{@specs[sym].long} must be specified" unless given_args.include? sym
  end

  ## parse parameters
  given_args.each do |sym, given_data|
    arg, params, negative_given = given_data.values_at :arg, :params, :negative_given

    opts = @specs[sym]
    if params.empty? && !opts.flag?
      raise CommandlineError, "option '#{arg}' needs a parameter" unless opts.default

      params << (opts.array_default? ? opts.default.clone : [opts.default])
    end

    vals["#{sym}_given".intern] = true # mark argument as specified on the commandline

    vals[sym] = opts.parse(params, negative_given)

    if opts.single_arg?
      vals[sym] = if opts.multi? # multiple options, each with a single parameter
                    vals[sym].map { |p| p[0] }
                  else # single parameter
                    vals[sym][0][0]
                  end
    elsif opts.multi_arg? && !opts.multi?
      vals[sym] = vals[sym][0] # single option, with multiple parameters
    end
    # else: multiple options, with multiple parameters

    opts.callback.call(vals[sym]) if opts.callback
  end

  ## modify input in place with only those
  ## arguments we didn't process
  cmdline.clear
  @leftovers.each { |l| cmdline << l }

  ## allow openstruct-style accessors
  class << vals

    def method_missing(m, *_args)
      self[m] || self[m.to_s]
    end

  end
  vals
end

#stop_on(*words) ⇒ Object

Defines a set of words which cause parsing to terminate when encountered, such that any options to the left of the word are parsed as usual, and options to the right of the word are left intact.

A typical use case would be for subcommand support, where these would be set to the list of subcommands. A subsequent Optimist invocation would then be used to parse subcommand options, after shifting the subcommand off of ARGV.



290
291
292
# File 'lib/optimist_with_insert_blanks.rb', line 290

def stop_on(*words)
  @stop_words = [*words].flatten
end

#stop_on_unknownObject

Similar to #stop_on, but stops on any unknown word when encountered (unless it is a parameter for an argument). This is useful for cases where you don’t know the set of subcommands ahead of time, i.e., without first parsing the global options.



304
305
306
# File 'lib/optimist_with_insert_blanks.rb', line 304

def stop_on_unknown
  @stop_on_unknown = true
end

#synopsis(s = nil) ⇒ Object

Adds a synopsis (command summary description) right below the usage line, or as the first line if usage isn’t specified.



249
250
251
# File 'lib/optimist_with_insert_blanks.rb', line 249

def synopsis(s = nil)
  s ? @synopsis = s : @synopsis
end

#usage(s = nil) ⇒ Object

Sets the usage string. If set the message will be printed as the first line in the help (educate) output and ending in two new lines.



243
244
245
# File 'lib/optimist_with_insert_blanks.rb', line 243

def usage(s = nil)
  s ? @usage = s : @usage
end

#version(s = nil) ⇒ Object

Sets the version string. If set, the user can request the version on the commandline. Should probably be of the form “<program name> <version number>”.



236
237
238
# File 'lib/optimist_with_insert_blanks.rb', line 236

def version(s = nil)
  s ? @version = s : @version
end

#widthObject

:nodoc:



500
501
502
503
504
505
506
507
508
509
510
511
512
# File 'lib/optimist_with_insert_blanks.rb', line 500

def width # :nodoc:
  @width ||= if $stdout.tty?
               begin
                 require 'io/console'
                 w = IO.console.winsize.last
                 w.to_i > 0 ? w : 80
               rescue LoadError, NoMethodError, Errno::ENOTTY, Errno::EBADF, Errno::EINVAL
                 legacy_width
               end
             else
               80
             end
end

#wrap(str, opts = {}) ⇒ Object

:nodoc:



522
523
524
525
526
527
528
529
530
531
532
533
# File 'lib/optimist_with_insert_blanks.rb', line 522

def wrap(str, opts = {}) # :nodoc:
  if str == ''
    ['']
  else
    inner = false
    str.split("\n").map do |s|
      line = wrap_line s, opts.merge(inner: inner)
      inner = true
      line
    end.flatten
  end
end