Module: Xolo::Core::Constants

Included in:
Xolo
Defined in:
lib/xolo/core/constants.rb

Overview

Constants useful throughout Xolo

Constant Summary collapse

BLANK =

Empty strings are used in various places

''
NONE =

The value to use when unsetting an option

'none'
OK =
'OK'
ERROR =
'ERROR'
X =

Several things use x

'x'
DASH =

CLI options and other things use dashes

'-'
DOT =

Several things use dots

'.'
CANCEL =

Cancelling is often an option

'Cancel'
DOTAPP =

and we check for things ending with .app

'.app'
DOTJSON =
'.json'
TRUE_FALSE =

These are handy for testing values without making new arrays, strings, etc every time.

[true, false].freeze
COMMA_SEP_RE =

lots of things get split on commmas

/\s*,\s*/.freeze
COMMA_JOIN =

lots of things get joined with commas

', '
SEMICOLON_SEP_RE =

Some things get split on semicolons

/\s*;\s*/.freeze
ITEM_UPLOADED =

Once a thing has been uploaded and saved, this is what the server returns as the attr value

'uploaded'
DOT_PKG =
'.pkg'
UNKNOWN =
'unknown'
OK_PKG_EXTS =

Installer packages must have one of these extensions

[DOT_PKG]
TARGET_ALL =

The value to use when all computers are the release-targets and for all manual-install policies

'all'
MANAGED =

Title Types

:managed
SUBSCRIBED =
:subscribed
TITLE_TYPES =
[MANAGED, SUBSCRIBED].freeze
DEFAULT_TITLE_TYPE =
:managed

Class Method Summary collapse

Class Method Details

.included(includer) ⇒ Object

when this module is included



77
78
79
# File 'lib/xolo/core/constants.rb', line 77

def self.included(includer)
  Xolo.verbose_include includer, self
end