Module: Xolo::Server::Constants

Included in:
Xolo::Server
Defined in:
lib/xolo/server/constants.rb

Constant Summary collapse

EXECUTABLE_FILENAME =

Constants

'xoloserver'
APP_ENV_DEV =

Sinatra App Environments

'development'
APP_ENV_TEST =
'test'
APP_ENV_PROD =
'production'
SESSION_EXPIRE_AFTER =

Sinatra App Settings

3600
DATA_DIR =

Paths

Pathname.new('/Library/Application Support/xoloserver')
BACKUPS_DIR =
DATA_DIR + 'backups'
PROGRESS_COMPLETE =

streaming progress from the server. When a line containing only this string shows up in a stream file that means the stream is done, and no more lines will be sent.

'PROGRESS_COMPLETE'
MAX_JAMF_WAIT_FOR_TITLE_EDITOR =

The max time (in seconds) to wait for a the Jamf server to see a change in the Title Editor, e.g. a new version appearing or an EA needing acceptance. Normally the Jamf server will check in with the Title Editor every 5 minutes.

3600
MAX_JAMF_WAIT_FOR_PKG_DELETION =

The max time (in seconds) to wait for a the Jamf server to stop the pkg deletion thread pool. It will wait until the queue is empty, or until this time has passed. Each pkg deletion thread can take up to 5 minutes, and there are 10 threads in the pool.

3600
JAMF_OBJECT_NAME_PFX =

Jamf objects are named with this prefix followed by -<version> See also: Xolo::Server::Version#jamf_obj_name_pfx which holds the full prefix for that version, and is used as the full object name if appropriate (e.g. Package objects)</p> </div> </div> <div class="tags"> </div> </dt> <dd><pre class="code"><span class='tstring'><span class='tstring_beg'>'</span><span class='tstring_content'>xolo-</span><span class='tstring_end'>'</span></span></pre></dd> <dt id="JAMF_TEST_OBJECT_NAME_PFX-constant" class="">JAMF_TEST_OBJECT_NAME_PFX = <div class="docstring"> <div class="discussion"> <p>Jamf objects from a test server are named with this prefix followed by <title>-<version> See also: Xolo::Server::Configuration</p> </div> </div> <div class="tags"> </div> </dt> <dd><pre class="code"><span class='tstring'><span class='tstring_beg'>'</span><span class='tstring_content'>xolotest-</span><span class='tstring_end'>'</span></span></pre></dd> <dt id="WEBHOOK_HANDLER_ADMIN_USERNAME-constant" class="">WEBHOOK_HANDLER_ADMIN_USERNAME = <div class="docstring"> <div class="discussion"> <p>when processing things via Jamf webhooks, this is the <a href=":admin">session</a></p> </div> </div> <div class="tags"> </div> </dt> <dd><pre class="code"><span class='tstring'><span class='tstring_beg'>'</span><span class='tstring_content'>xolo-webhook-handler</span><span class='tstring_end'>'</span></span></pre></dd> <dt id="JPAPI_PATCH_TITLE_ENDPOINT-constant" class="">JPAPI_PATCH_TITLE_ENDPOINT = <div class="docstring"> <div class="discussion"> <p>TODO: remove this stuff when ruby-jss supports Patch Titles via JPAPI</p> </div> </div> <div class="tags"> </div> </dt> <dd><pre class="code"><span class='tstring'><span class='tstring_beg'>'</span><span class='tstring_content'>v2/patch-software-title-configurations</span><span class='tstring_end'>'</span></span></pre></dd> </dl> <h2> Class Method Summary <small><a href="#" class="summary_toggle">collapse</a></small> </h2> <ul class="summary"> <li class="public "> <span class="summary_signature"> <a href="#included-class_method" title="included (class method)">.<strong>included</strong>(includer) ⇒ Object </a> </span> <span class="summary_desc"><div class='inline'><p>when this module is included.</p></div></span> </li> </ul> <div id="class_method_details" class="method_details_list"> <h2>Class Method Details</h2> <div class="method_details first"> <h3 class="signature first" id="included-class_method"> .<strong>included</strong>(includer) ⇒ <tt>Object</tt> </h3><div class="docstring"> <div class="discussion"> <p>when this module is included</p> </div> </div> <div class="tags"> </div><table class="source_code"> <tr> <td> <pre class="lines"> 16 17 18</pre> </td> <td> <pre class="code"><span class="info file"># File 'lib/xolo/server/constants.rb', line 16</span> <span class='kw'>def</span> <span class='kw'>self</span><span class='period'>.</span><span class='id identifier rubyid_included'>included</span><span class='lparen'>(</span><span class='id identifier rubyid_includer'>includer</span><span class='rparen'>)</span> <span class='const'><span class='object_link'><a href="../../Xolo.html" title="Xolo (module)">Xolo</a></span></span><span class='period'>.</span><span class='id identifier rubyid_verbose_include'>verbose_include</span> <span class='id identifier rubyid_includer'>includer</span><span class='comma'>,</span> <span class='kw'>self</span> <span class='kw'>end</span></pre> </td> </tr> </table> </div> </div> </div> <div id="footer"> Generated on Sat Jul 25 19:07:54 2026 by <a href="https://yardoc.org" title="Yay! A Ruby Documentation Tool" target="_parent">yard</a> 0.9.44 (ruby-4.0.2). </div> </div> </body> </html>